"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Cache_1 = require("./Cache");
var ApiService_1 = require("./ApiService");
var ServiceBase = /** @class */ (function () {
    function ServiceBase(_config) {
        this.config = _config;
        this.cache = new Cache_1.InMemoryCache();
        if (_config.cache)
            this.cache = _config.cache;
    }
    /**
     * Fetch environment data from the server
     * Check if the data has been modified and if not return from cache
     * @param callback
     */
    ServiceBase.prototype.fetchRemote = function (callback) {
        var _this = this;
        var requestHeaders = [
            {
                key: "Accept",
                value: "application/json"
            },
            {
                key: "Content-Type",
                value: "application/json"
            }
        ];
        var api = new ApiService_1.ApiService().setHeaders(requestHeaders);
        var url = this.config.buildUrl();
        // Check to see if we have a current ETag
        var etag = this.cache.Get("ETag");
        if (etag) {
            api.setHeaders([{ key: "If-None-Match", value: "" + etag }]);
        }
        api.get(url)
            .then(function (response) {
            if (response.status == 304) { // read cache
                var json_1 = _this.cache.Get(_this.config.sdkKey);
                callback(json_1);
                return;
            }
            var responseHeaders = response.headers;
            etag = responseHeaders.get("etag");
            if (typeof (etag) === "string") {
                etag = etag.replace(/['"]+/g, '');
            }
            _this.cache.Set("ETag", etag);
            var json = response.parsedBody;
            if (_this.validateJson(json)) {
                _this.cache.Set(_this.config.sdkKey, json);
                callback(json);
            }
        })
            .catch(function (error) {
            // return cache if available
            var json = _this.cache.Get(_this.config.sdkKey);
            if (json) {
                callback(json);
            }
            throw error;
        });
    };
    ServiceBase.prototype.validateJson = function (json) {
        return true;
    };
    return ServiceBase;
}());
exports.ServiceBase = ServiceBase;
